@@ -151,7 +151,7 @@ def maybe_clean_cache():
     if os.path.exists(cachedir):
         for fname in os.listdir(cachedir):
             path = os.path.join(cachedir, fname)
-            if time.time() > os.path.getmtime(path) + (60 * 60 * Config['GLOBAL'].getint('clean_cache_after', 12)):
+            if time.time() > os.path.getmtime(path) + (60 * 60 * Config['GLOBAL'].getint('clean_cache_after', 48)):
                 # Remove the whole directory if necessary
                 if os.path.isdir(path):
                     import shutil
@@ -212,7 +212,7 @@ def get_cached(keyword, search_engine, scrapemode):
         except FileNotFoundError as err:
             return False
 
-        if (time.time() - modtime) / 60 / 60 > Config['GLOBAL'].getint('clean_cache_after', 12):
+        if (time.time() - modtime) / 60 / 60 > Config['GLOBAL'].getint('clean_cache_after', 48):
             return False
 
         path = os.path.join(cdir, fname)
@@ -344,7 +344,7 @@ def _caching_is_one_to_one(keywords, search_engine, scrapemode):
         return True
 
 
-def parse_all_cached_files(keywords, session, scraper_search, try_harder=False):
+def parse_all_cached_files(keywords, search_engines, session, scraper_search, try_harder=False):
     """Walk recursively through the cachedir (as given by the Config) and parse all cached files.
 
     Args:
@@ -357,21 +357,24 @@ def parse_all_cached_files(keywords, session, scraper_search, try_harder=False):
         A list of keywords that couldn't be parsed and which need to be scraped anew.
     """
     google_query_needle = re.compile(r'<title>(?P<kw>.*?) - Google Search</title>')
+
     files = _get_all_cache_files()
     mapping = {}
-    search_engine = Config['SCRAPING'].get('search_engine')
     scrapemethod = Config['SCRAPING'].get('scrapemethod')
+    num_cached = 0
+
     for kw in keywords:
-        key = cached_file_name(kw, search_engine, scrapemethod)
+        for search_engine in search_engines:
+            key = cached_file_name(kw, search_engine, scrapemethod)
 
-        out('Params(keyword="{kw}", search_engine="{se}", scrapemethod="{sm}" yields {hash}'.format(
-                kw=kw,
-                se=search_engine,
-                sm=scrapemethod,
-                hash=key
-            ), lvl=5)
+            out('Params(keyword="{kw}", search_engine="{se}", scrapemethod="{sm}" yields {hash}'.format(
+                    kw=kw,
+                    se=search_engine,
+                    sm=scrapemethod,
+                    hash=key
+                ), lvl=5)
 
-        mapping[key] = kw
+            mapping[key] = (kw, search_engine)
 
     for path in files:
         # strip of the extension of the path if it has eny
@@ -381,14 +384,17 @@ def parse_all_cached_files(keywords, session, scraper_search, try_harder=False):
             if fname.endswith(ext):
                 clean_filename = fname.rstrip('.' + ext)
 
-        query = mapping.get(clean_filename, None)
+        query = search_engine = None
+        val = mapping.get(clean_filename, None)
+        if val:
+            query, search_engine = val
 
-        if query:
+        if query and search_engine:
             # We found a file that contains the keyword, search engine name and
             # searchmode that fits our description. Let's see if there is already
             # an record in the database and link it to our new ScraperSearch object.
             try:
-                serp =  session.query(SearchEngineResultsPage).filter(
+                serp = session.query(SearchEngineResultsPage).filter(
                         SearchEngineResultsPage.query == query,
                         SearchEngineResultsPage.search_engine_name == search_engine,
                         SearchEngineResultsPage.scrapemethod == scrapemethod).one()
@@ -397,8 +403,9 @@ def parse_all_cached_files(keywords, session, scraper_search, try_harder=False):
                 # we have a cache file that matches the above identifying information
                 # but it was never stored to the database.
                 logger.error('No entry for file {} found in database. Will parse again.'.format(clean_filename))
+                html = read_cached_file(get_path(fname))
                 serp = parse_serp(
-                    html=read_cached_file(get_path(fname)),
+                    html=html,
                     search_engine=search_engine,
                     scrapemethod=scrapemethod,
                     current_page=0,
@@ -406,22 +413,21 @@ def parse_all_cached_files(keywords, session, scraper_search, try_harder=False):
                 )
             except MultipleResultsFound as e:
                 raise e
-            finally:
+
+            if serp:
                 scraper_search.serps.append(serp)
 
             mapping.pop(clean_filename)
-
-        # TODO: support query detection for all supported search engines
-        # by parsing the keyword, search engine from the raw html
+            num_cached += 1
 
     out('{} cache files found in {}'.format(len(files), Config['GLOBAL'].get('cachedir')), lvl=1)
     out('{}/{} keywords have been cached and are ready to get parsed. {} remain to get scraped.'.format(
-        len(keywords) - len(mapping), len(keywords), len(mapping)), lvl=1)
+        num_cached, len(keywords), len(keywords) - num_cached), lvl=1)
 
     session.add(scraper_search)
     session.commit()
     # return the remaining keywords to scrape
-    return mapping.values()
+    return [e[0] for e in mapping.values()]
 
 
 def fix_broken_cache_names(url, search_engine, scrapemode):
