@@ -8,13 +8,7 @@ import lxml.html
 import logging
 import pprint
 from GoogleScraper.database import SearchEngineResultsPage, Link
-
-try:
-    from cssselect import HTMLTranslator, SelectorError
-    from bs4 import UnicodeDammit
-except ImportError as ie:
-    if hasattr(ie, 'name') and ie.name == 'bs4' or hasattr(ie, 'args') and 'bs4' in str(ie):
-        sys.exit('Install bs4 with the command "sudo pip3 install beautifulsoup4"')
+from cssselect import HTMLTranslator, SelectorError
 
 logger = logging.getLogger('GoogleScraper')
 
@@ -76,6 +70,7 @@ class Parser():
         self.searchtype = searchtype
         self.dom = None
         self.search_results = {}
+        self.search_results['num_results'] = ''
         
         if self.html:
             self.parse()
@@ -94,9 +89,6 @@ class Parser():
         # Apply subclass specific behaviour after parsing has happened
         self.after_parsing()
 
-        # push the data to the database
-
-
     def _parse(self):
         """Internal parse the dom according to the provided css selectors.
         
@@ -104,72 +96,82 @@ class Parser():
         """
         
         # Try to parse the provided HTML string using lxml
-        doc = UnicodeDammit(self.html, is_html=True)
-        parser = lxml.html.HTMLParser(encoding=doc.declared_html_encoding)
-        self.dom = lxml.html.document_fromstring(self.html, parser=parser)
-        self.dom.resolve_base_href()
+        try:
+            parser = lxml.html.HTMLParser(encoding='utf-8')
+            self.dom = lxml.html.document_fromstring(self.html, parser=parser)
+            self.dom.resolve_base_href()
+        except Exception as e:
+            # maybe wrong encoding
+            logger.error(e)
         
         # try to parse the number of results.
         attr_name = self.searchtype + '_search_selectors'
         selector_dict = getattr(self, attr_name, None)
-        
+
         # short alias because we use it so extensively
         css_to_xpath = HTMLTranslator().css_to_xpath
-        
+
         # get the appropriate css selectors for the num_results for the keyword
         num_results_selector = getattr(self, 'num_results_search_selectors', None)
         self.search_results['num_results'] = ''
-        if num_results_selector:
-            try:
-                self.search_results['num_results'] = self.dom.xpath(css_to_xpath(num_results_selector))[0].text_content()
-            except IndexError as e:
-                logger.warning('Cannot parse num_results from serp page')
 
-        if not selector_dict:
+        if isinstance(num_results_selector, list) and num_results_selector:
+            for selector in num_results_selector:
+                try:
+                    self.search_results['num_results'] = self.dom.xpath(css_to_xpath(selector))[0].text_content()
+                except IndexError as e:
+                    logger.warning('Cannot parse num_results from serp page with selector {}'.format(selector))
+                else: # leave when first selector grabbed something
+                    break
+
+        if not selector_dict and not isinstance(selector_dict, dict):
             raise InvalidSearchTypeExcpetion('There is no such attribute: {}. No selectors found'.format(attr_name))
-            
-        for result_type, selectors in selector_dict.items():
+
+        for result_type, selector_class in selector_dict.items():
+
             self.search_results[result_type] = []
-            
-            results = self.dom.xpath(
-                css_to_xpath('{container} {result_container}'.format(**selectors))
-            )
-            
-            to_extract = set(selectors.keys()) - {'container', 'result_container'}                
-            selectors_to_use = dict(((key, selectors[key]) for key in to_extract if key in selectors.keys()))
-            
-            for index, result in enumerate(results):
-                # Let's add primitive support for CSS3 pseudo selectors
-                # We just need two of them
-                # ::text
-                # ::attr(someattribute)
-                
-                # You say we should use xpath expresssions instead?
-                # Maybe you're right, but they are complicated when it comes to classes,
-                # have a look here: http://doc.scrapy.org/en/latest/topics/selectors.html
-                serp_result = {}
-                for key, selector in selectors_to_use.items():
-                    value = None
-                    if selector.endswith('::text'):
-                        try:
-                            value = result.xpath(css_to_xpath(selector.split('::')[0]))[0].text_content()
-                        except IndexError as e:
-                            pass
-                    else:
-                        attr = re.search(r'::attr\((?P<attr>.*)\)$', selector).group('attr')
-                        if attr:
+
+            for selector_specific, selectors in selector_class.items():
+
+                results = self.dom.xpath(
+                    css_to_xpath('{container} {result_container}'.format(**selectors))
+                )
+
+                to_extract = set(selectors.keys()) - {'container', 'result_container'}
+                selectors_to_use = {key: selectors[key] for key in to_extract if key in selectors.keys()}
+
+                for index, result in enumerate(results):
+                    # Let's add primitive support for CSS3 pseudo selectors
+                    # We just need two of them
+                    # ::text
+                    # ::attr(someattribute)
+
+                    # You say we should use xpath expresssions instead?
+                    # Maybe you're right, but they are complicated when it comes to classes,
+                    # have a look here: http://doc.scrapy.org/en/latest/topics/selectors.html
+                    serp_result = {}
+                    for key, selector in selectors_to_use.items():
+                        value = None
+                        if selector.endswith('::text'):
                             try:
-                                value = result.xpath(css_to_xpath(selector.split('::')[0]))[0].get(attr)
+                                value = result.xpath(css_to_xpath(selector.split('::')[0]))[0].text_content()
                             except IndexError as e:
                                 pass
                         else:
-                            try:
-                                value = result.xpath(css_to_xpath(selector))[0].text_content()
-                            except IndexError as e:
-                                pass
-                    serp_result[key] = value
-                if serp_result:
-                    self.search_results[result_type].append(serp_result)
+                            attr = re.search(r'::attr\((?P<attr>.*)\)$', selector).group('attr')
+                            if attr:
+                                try:
+                                    value = result.xpath(css_to_xpath(selector.split('::')[0]))[0].get(attr)
+                                except IndexError as e:
+                                    pass
+                            else:
+                                try:
+                                    value = result.xpath(css_to_xpath(selector))[0].text_content()
+                                except IndexError as e:
+                                    pass
+                        serp_result[key] = value
+                    if serp_result:
+                        self.search_results[result_type].append(serp_result)
 
     def clean_html(self, html):
         """Clean the html from any bloated data that is of no interest.
@@ -216,6 +218,14 @@ then you're adding a new search type with the name finance.
 
 Each class needs a attribute called num_results_search_selectors, that
 extracts the number of searches that were found by the keyword.
+
+Please note:
+The actual selectors are wrapped in a dictionary to clarify with which IP
+they were requested. The key to the wrapper div allows to specify distinct
+criteria to whatever settings you used when you requested the page. So you
+might add your own selectors for different User-Agents, distinct HTTP headers, what-
+ever you may imagine. This allows the most dynamic parsing behaviour and makes
+it very easy to grab all data the site has to offer.
 """
 
 
@@ -224,24 +234,47 @@ class GoogleParser(Parser):
     
     search_types = ['normal', 'image']
     
-    num_results_search_selectors = '#resultStats'
+    num_results_search_selectors = ['#resultStats']
     
     normal_search_selectors = {
         'results': {
-            'container': '#center_col',
-            'result_container': 'li.g ',
-            'link': 'h3.r > a:first-child::attr(href)',
-            'snippet': 'div.s span.st::text',
-            'title': 'h3.r > a:first-child::text',
-            'visible_link': 'cite::text'
+            'us_ip': {
+                'container': '#center_col',
+                'result_container': 'li.g ',
+                'link': 'h3.r > a:first-child::attr(href)',
+                'snippet': 'div.s span.st::text',
+                'title': 'h3.r > a:first-child::text',
+                'visible_link': 'cite::text'
+            },
+            'de_ip': {
+                'container': '#center_col',
+                'result_container': 'li.g ',
+                'link': 'h3.r > a:first-child::attr(href)',
+                'snippet': 'div.s span.st::text',
+                'title': 'h3.r > a:first-child::text',
+                'visible_link': 'cite::text'
+            }
         },
-        'ads_main' : {
-            'container': '#center_col',
-            'result_container': 'li.ads-ad',
-            'link': 'h3.r > a:first-child::attr(href)',
-            'snippet': 'div.s span.st::text',
-            'title': 'h3.r > a:first-child::text',
-            'visible_link': '.ads-visurl cite::text',
+        'ads_main': {
+            'us_ip': {
+                'container': '#center_col',
+                'result_container': 'li.ads-ad',
+                'link': 'h3.r > a:first-child::attr(href)',
+                'snippet': 'div.s span.st::text',
+                'title': 'h3.r > a:first-child::text',
+                'visible_link': '.ads-visurl cite::text',
+            },
+            'de_ip': {
+                'container': '#center_col',
+                'result_container': '.ads-ad',
+                'link': 'h3 > a:first-child::attr(href)',
+                'snippet': '.ads-creative::text',
+                'title': 'h3 > a:first-child::text',
+                'visible_link': '.ads-visurl cite::text',
+            }
+        },
+        'ads_aside': {
+
         }
     }
     
@@ -280,17 +313,19 @@ class YandexParser(Parser):
 
     search_types = ['normal']
     
-    num_results_search_selectors = None
+    num_results_search_selectors = []
     
     normal_search_selectors = {
         'results': {
-            'container': 'div.serp-list',
-            'result_container': 'div.serp-item__wrap ',
-            'link': 'a.serp-item__title-link::attr(href)',
-            'snippet': 'div.serp-item__text::text',
-            'title': 'a.serp-item__title-link::text',
-            'visible_link': 'a.serp-url__link::attr(href)'
-        },
+            'de_ip': {
+                'container': 'div.serp-list',
+                'result_container': 'div.serp-item__wrap ',
+                'link': 'a.serp-item__title-link::attr(href)',
+                'snippet': 'div.serp-item__text::text',
+                'title': 'a.serp-item__title-link::text',
+                'visible_link': 'a.serp-url__link::attr(href)'
+            }
+        }
     }
     
     
@@ -299,24 +334,44 @@ class BingParser(Parser):
     
     search_types = ['normal']
     
-    num_results_search_selectors = '.sb_count'
+    num_results_search_selectors = ['.sb_count']
     
     normal_search_selectors = {
         'results': {
-            'container': 'ol#b_results',
-            'result_container': 'li.b_algo',
-            'link': '.b_title > h2 > a::attr(href)',
-            'snippet': '.b_snippet > p::text',
-            'title': '.b_title > h2 > a::text',
-            'visible_link': 'cite::text'
+            'us_ip': {
+                'container': '#b_results',
+                'result_container': '.b_algo',
+                'link': 'h2 > a::attr(href)',
+                'snippet': '.b_caption > .b_attribution > p::text',
+                'title': 'h2::text',
+                'visible_link': 'cite::text'
+            },
+            'de_ip': {
+                'container': '#b_results',
+                'result_container': '.b_algo',
+                'link': 'h2 > a::attr(href)',
+                'snippet': '.b_caption > p::text',
+                'title': 'h2::text',
+                'visible_link': 'cite::text'
+            }
         },
-        'ads_main' : {
-            'container': 'ol#b_results',
-            'result_container': 'li.b_ad',
-            'link': '.sb_add > h2 > a::attr(href)',
-            'snippet': '.b_caption::text',
-            'title': '.sb_add > h2 > a::text',
-            'visible_link': 'cite::text'
+        'ads_main': {
+            'us_ip': {
+                'container': '#b_results .b_ad',
+                'result_container': '.sb_add',
+                'link': 'h2 > a::attr(href)',
+                'snippet': '.sb_addesc::text',
+                'title': 'h2 > a::text',
+                'visible_link': 'cite::text'
+            },
+            'de_ip': {
+                'container': '#b_results .b_ad',
+                'result_container': '.sb_add',
+                'link': 'h2 > a::attr(href)',
+                'snippet': '.b_caption > p::text',
+                'title': 'h2 > a::text',
+                'visible_link': 'cite::text'
+            }
         }
     }
 
@@ -326,16 +381,18 @@ class YahooParser(Parser):
     
     search_types = ['normal']
     
-    num_results_search_selectors = '#pg > span:last-child'
+    num_results_search_selectors = ['#pg > span:last-child']
     
     normal_search_selectors = {
         'results': {
-            'container': '#main',
-            'result_container': '.res',
-            'link': 'div > h3 > a::attr(href)',
-            'snippet': 'div.abstr::text',
-            'title': 'div > h3 > a::text',
-            'visible_link': 'span.url::text'
+            'de_ip': {
+                'container': '#main',
+                'result_container': '.res',
+                'link': 'div > h3 > a::attr(href)',
+                'snippet': 'div.abstr::text',
+                'title': 'div > h3 > a::text',
+                'visible_link': 'span.url::text'
+            }
         },
     }
     
@@ -345,16 +402,18 @@ class BaiduParser(Parser):
     
     search_types = ['normal']
     
-    num_results_search_selectors = '#container .nums'
+    num_results_search_selectors = ['#container .nums']
     
     normal_search_selectors = {
         'results': {
-            'container': '#content_left',
-            'result_container': '.result-op',
-            'link': 'h3 > a.t::attr(href)',
-            'snippet': '.c-abstract::text',
-            'title': 'h3 > a.t::text',
-            'visible_link': 'span.c-showurl::text'
+            'de_ip': {
+                'container': '#content_left',
+                'result_container': '.result-op',
+                'link': 'h3 > a.t::attr(href)',
+                'snippet': '.c-abstract::text',
+                'title': 'h3 > a.t::text',
+                'visible_link': 'span.c-showurl::text'
+            }
         },
     }
 
@@ -364,16 +423,18 @@ class DuckduckgoParser(Parser):
     
     search_types = ['normal']
     
-    num_results_search_selectors = None
+    num_results_search_selectors = []
     
     normal_search_selectors = {
         'results': {
-            'container': '#links',
-            'result_container': '.result',
-            'link': '.result__title > a::attr(href)',
-            'snippet': 'result__snippet::text',
-            'title': '.result__title > a::text',
-            'visible_link': '.result__url__domain::text'
+            'de_ip': {
+                'container': '#links',
+                'result_container': '.result',
+                'link': '.result__title > a::attr(href)',
+                'snippet': 'result__snippet::text',
+                'title': '.result__title > a::text',
+                'visible_link': '.result__url__domain::text'
+            }
         },
     }
 
