@@ -23,7 +23,7 @@ except ImportError as ie:
 
 import GoogleScraper.socks as socks
 from GoogleScraper.caching import get_cached, cache_results, cached_file_name, cached
-from GoogleScraper.database import SearchEngineResultsPage, Link
+from GoogleScraper.database import SearchEngineResultsPage, Link, get_session
 from GoogleScraper.config import Config
 from GoogleScraper.log import out
 from GoogleScraper.parsing import GoogleParser, YahooParser, YandexParser, BaiduParser, BingParser, DuckduckgoParser, get_parser_by_search_engine
@@ -107,17 +107,15 @@ class SearchEngineScrape(metaclass=abc.ABCMeta):
     sophisticated input format and some more tricky engineering.
     """
 
-    def __init__(self, keywords=None, session=None, scraper_search=None, db_lock=None, cache_lock=None,
+    def __init__(self, keywords=None, scraper_search=None, session=None, db_lock=None, cache_lock=None,
                  start_page_pos=1, search_engine=None, search_type=None, proxy=None):
         """Instantiate an SearchEngineScrape object.
 
         Args:
             TODO
         """
-        if not search_engine:
-            self.search_engine = Config['SCRAPING'].get('search_engine', 'google')
-        else:
-            self.search_engine = search_engine
+        self.search_engine = search_engine
+        assert self.search_engine, 'You need to specify an search_engine'
 
         self.search_engine = self.search_engine.lower()
 
@@ -165,9 +163,6 @@ class SearchEngineScrape(metaclass=abc.ABCMeta):
         else:
             self.ip = '127.0.0.1'
 
-        # set the database scoped session
-        self.session = session
-
         # the scraper_search object
         self.scraper_search = scraper_search
         
@@ -181,6 +176,9 @@ class SearchEngineScrape(metaclass=abc.ABCMeta):
         # init the cache lock
         self.cache_lock = cache_lock
 
+        # set the session
+        self.session = session
+
 
     @abc.abstractmethod
     def search(self, *args, **kwargs):
@@ -229,26 +227,22 @@ class SearchEngineScrape(metaclass=abc.ABCMeta):
 
     def store(self):
         """Store the parsed data in the sqlalchemy scoped session."""
-        assert self.session, 'You need to pass a sqlalchemy scoped session to SearchEngineScrape instances'
-
-        num_results = 0
-
-        ip = '127.0.0.1'
-
-        serp = SearchEngineResultsPage(
-            search_engine_name=self.search_engine,
-            scrapemethod=self.scrapemethod,
-            page_number=self.current_page,
-            requested_at=datetime.datetime.utcnow(),
-            requested_by=ip,
-            query=self.current_keyword,
-            num_results_for_keyword=self.parser.search_results['num_results'],
-        )
-        self.scraper_search.serps.append(serp)
-        self.session.add(self.scraper_search)
-        self.session.commit()
+        assert self.session, 'No database session. Turning down.'
 
         with self.db_lock:
+            num_results = 0
+
+            serp = SearchEngineResultsPage(
+                search_engine_name=self.search_engine,
+                scrapemethod=self.scrapemethod,
+                page_number=self.current_page,
+                requested_at=datetime.datetime.utcnow(),
+                requested_by=self.ip,
+                query=self.current_keyword,
+                num_results_for_keyword=self.parser.search_results['num_results'],
+            )
+            self.scraper_search.serps.append(serp)
+
             for key, value in self.parser.search_results.items():
                 if isinstance(value, list):
                     rank = 1
@@ -349,6 +343,10 @@ class HttpScrape(SearchEngineScrape, threading.Timer):
         if Config['SCRAPING'].getboolean('check_proxies'):
             self.proxy_check()
 
+        out('[+] HttpScrape[{}] created using the search engine {}. Number of keywords to scrape={}, using proxy={}, number of pages={}'.format(
+            self.ip, self.search_engine, len(self.keywords), self.proxy, self.num_pages_per_keyword), lvl=1)
+
+
     def set_proxy(self):
         """Setup a socks connection for the socks module bound to this instance.
 
@@ -464,12 +462,12 @@ class HttpScrape(SearchEngineScrape, threading.Timer):
 
         if not html:
             try:
-                if Config['GLOBAL'].getint('verbosity', 0) > 1:
-                    logger.info('[HTTP] Base_url: {base_url}, headers={headers}, params={params}'.format(
-                        base_url=self.base_search_url,
-                        headers=self.headers,
-                        params=self.search_params)
-                    )
+                out('[HTTP - {proxy}] Base_url: {base_url}, headers={headers}, params={params}'.format(
+                    proxy=self.proxy,
+                    base_url=self.base_search_url,
+                    headers=self.headers,
+                    params=self.search_params),
+                lvl=3)
 
                 request = self.requests.get(self.base_search_url, headers=self.headers,
                                  params=self.search_params, timeout=3.0)
@@ -527,21 +525,21 @@ class SelScrape(SearchEngineScrape, threading.Thread):
         self.browser_type = Config['SELENIUM'].get('sel_browser', 'chrome').lower()
         self.browser_num = browser_num
         self.captcha_lock = captcha_lock
-        self.ip = '127.0.0.1'
         self.search_number = 0
         self.scrapemethod = 'selenium'
 
         # get the base search url based on the search engine.
         self.base_search_url = get_base_search_url_by_search_engine(self.search_engine, self.scrapemethod)
 
-        # How long to sleep (ins seconds) after every n-th request
+        # How long to sleep (in seconds) after every n-th request
         self.sleeping_ranges = dict()
         for line in Config['SELENIUM'].get('sleeping_ranges').split('\n'):
-            assert line.count(';') == 1
-            key, value = line.split(';')
+            assert line.count(':') == 1, 'Invalid sleep range format.'
+            key, value = line.split(':')
             self.sleeping_ranges[int(key)] = tuple([int(offset.strip()) for offset in value.split(',')])
 
-        out('[+] SelScraper[{}] created using the search engine {}. Number of keywords to scrape={}, using proxy={}, number of pages={}, browser_num={}'.format(self.search_engine, self.browser_type, len(self.keywords), self.proxy, self.num_pages_per_keyword, self.name), lvl=2)
+        out('[+] SelScrape[{}] created using the search engine {}. Number of keywords to scrape={}, using proxy={}, number of pages={}, browser_num={}'.format(
+            self.search_engine, self.browser_type, len(self.keywords), self.proxy, self.num_pages_per_keyword, self.name), lvl=1)
 
     def _largest_sleep_range(self, search_number):
         assert search_number >= 0
@@ -697,18 +695,16 @@ class SelScrape(SearchEngineScrape, threading.Thread):
         if Config['SCRAPING'].getboolean('check_proxies'):
             self.proxy_check()
 
-        url_params= {
-            'google': 'q={query}',
-            'yandex': 'text={query}',
-            'bing': 'q={query}',
-            'yahoo': 'p={query}',
-            'baidu': 'wd={query}',
-            'duckduckgo': 'q={query}'
+        starting_point = {
+            'google': 'https://www.google.com/',
+            'yandex': 'http://www.yandex.ru/',
+            'bing': 'http://www.bing.com/',
+            'yahoo': 'https://yahoo.com/',
+            'baidu': 'http://baidu.com/',
+            'duckduckgo': 'https://duckduckgo.com/'
         }[self.search_engine]
 
-        url_params = url_params.format(query=self.current_keyword)
-
-        self.starting_point = self.base_search_url +  url_params
+        self.starting_point = starting_point
 
         self.webdriver.get(self.starting_point)
 
@@ -774,12 +770,21 @@ class SelScrape(SearchEngineScrape, threading.Thread):
         """
         for self.current_keyword in self.keywords:
 
+            try:
+                self.search_input = WebDriverWait(self.webdriver, 5).until(
+                    EC.presence_of_element_located(self._get_search_input_field()))
+            except TimeoutException as e:
+                logger.error(e)
+                if not self.handle_request_denied():
+                    open('/tmp/out.png', 'wb').write(self.webdriver.get_screenshot_as_png())
+                    raise GoogleSearchError('search input field cannot be found.')
+
+            if self.search_input:
+                self.search_input.clear()
+                time.sleep(.25)
+                self.search_input.send_keys(self.current_keyword + Keys.ENTER)
+
             for self.current_page in range(1, self.num_pages_per_keyword + 1):
-                # match the largest sleep range
-                sleep_time = random.randrange(*self._largest_sleep_range(self.search_number))
-
-                time.sleep(sleep_time)
-
                 # Waiting until the keyword appears in the title may
                 # not be enough. The content may still be from the old page.
                 try:
@@ -787,6 +792,9 @@ class SelScrape(SearchEngineScrape, threading.Thread):
                 except TimeoutException as e:
                     logger.error(SeleniumSearchError('Keyword "{}" not found in title: {}'.format(self.current_keyword, self.webdriver.title)))
 
+                # match the largest sleep range
+                sleep_time = random.randrange(*self._largest_sleep_range(self.search_number))
+                time.sleep(sleep_time)
 
                 html = self.webdriver.page_source
 
@@ -807,20 +815,6 @@ class SelScrape(SearchEngineScrape, threading.Thread):
                     if not self.next_url:
                         break
 
-            try:
-                self.search_input = WebDriverWait(self.webdriver, 5).until(
-                    EC.presence_of_element_located(self._get_search_input_field()))
-            except TimeoutException as e:
-                logger.error(e)
-                if not self.handle_request_denied():
-                    open('/tmp/out.png', 'wb').write(self.webdriver.get_screenshot_as_png())
-                    raise GoogleSearchError('search input field cannot be found.')
-
-            if self.search_input:
-                self.search_input.clear()
-                time.sleep(.25)
-                self.search_input.send_keys(self.current_keyword + Keys.ENTER)
-
 
     def run(self):
         """Run the SelScraper."""
